/**
 * Create a CSS Custom Property behavior.
 * @param name The name of the CSS custom property
 * @param value The value or value resolver of the custom property
 * @param host A function to resolve the element to host the CSS custom property
 */
export function cssCustomPropertyBehaviorFactory(name, value, host) {
    return Object.freeze({
        name,
        value,
        host,
        bind(source) {
            const target = this.host(source);
            if (target !== null) {
                if (typeof target.registerCSSCustomProperty === "function") {
                    target.registerCSSCustomProperty(this);
                }
                else {
                    // There is potential for the custom property host element to not be
                    // constructed when this is run. We handle that case by accumulating
                    // the behaviors in a normal array. Behaviors associated this way will
                    // get registered when the host is connected
                    if (!Array.isArray(target.disconnectedCSSCustomPropertyRegistry)) {
                        target.disconnectedCSSCustomPropertyRegistry = [];
                    }
                    target.disconnectedCSSCustomPropertyRegistry.push(this);
                }
            }
        },
        unbind(source) {
            const target = this.host(source);
            if (target !== null) {
                target.unregisterCSSCustomProperty(this);
            }
        },
    });
}
