/**
 * Construct a behavior to conditionally apply a stylesheet based
 * on a MediaQueryList
 */
export function matchMediaStylesheetBehaviorFactory(query) {
    const cache = new WeakMap();
    return (sheet) => {
        return Object.freeze({
            query,
            cache,
            sheet,
            constructListener(source, sheet) {
                let attached = false;
                function listener() {
                    const { matches } = this;
                    if (matches && !attached) {
                        source.$fastController.addStyles(sheet);
                        attached = matches;
                    }
                    else if (!matches && attached) {
                        source.$fastController.removeStyles(sheet);
                        attached = matches;
                    }
                }
                return listener;
            },
            bind(source) {
                const { constructListener, query, cache } = this;
                const listener = constructListener(source, this.sheet);
                const cached = cache.get(source);
                // Invoke immediately to add if the query currently matches
                listener.bind(query)();
                query.addListener(listener);
                if (cached !== void 0) {
                    // Support multiple bindings of the same behavior
                    if (Array.isArray(cached)) {
                        cached.push(listener);
                    }
                    else {
                        cache.set(source, [cached, listener]);
                    }
                }
                else {
                    cache.set(source, listener);
                }
            },
            unbind(source) {
                const { cache, query } = this;
                const cached = cache.get(source);
                if (cached !== void 0) {
                    if (Array.isArray(cached)) {
                        cached.forEach(listener => query.removeListener(listener));
                    }
                    else {
                        query.removeListener(cached);
                    }
                    cache.delete(source);
                }
            },
        });
    };
}
/**
 * Applies ElementStyles to a FASTElement when the
 * forced-colors media query is matched, removes the
 * ElementStyles when the query is no-longer matched
 */
export const forcedColorsStylesheetBehavior = matchMediaStylesheetBehaviorFactory(window.matchMedia("(forced-colors)"));
